@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/connect.c 1.3 1997/01/22 20:18:55 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/connect.c 1.14 1997/03/18 20:03:15 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   Handle WWWOFFLE connections received by the demon.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -20,6 +20,7 @@
 #include <unistd.h>
 
 #include "wwwoffle.h"
+#include "config.h"
 #include "errors.h"
 
 
@@ -33,11 +34,15 @@ extern char *proxy;
 /*+ The online / offline status. +*/
 extern int online;
 
-/*+ The current number of active servers. +*/
-extern int n_servers;
+/*+ The current number active servers +*/
+extern int n_servers,           /*+ in total. +*/
+           n_fetch_servers;     /*+ fetching a previously requested page. +*/
 
-/*+ The wwwoffle client when fetching. +*/
-extern int fetching;
+/*+ The pids of the servers that are fetching. +*/
+extern int fetch_pids[MAX_FETCH_SERVERS];
+
+/*+ The wwwoffle client file descriptor when fetching. +*/
+extern int fetch_fd;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -53,39 +58,83 @@ void CommandConnect(FILE *client)
  if(!(line=fgets_realloc(line,client)))
    {PrintMessage(Warning,"Nothing to read from the wwwoffle control socket [%!s]."); return;}
 
+ if(strncmp(line,"WWWOFFLE ",9))
+   {
+    PrintMessage(Warning,"Not a WWWOFFLE command.");
+    return;
+   }
+
+ if(PassWord || !strncmp(&line[9],"PASSWORD ",9))
+   {
+    char *password;
+
+    if(strlen(line)<18)
+       password="";
+    else
+      {
+       int i;
+
+       for(i=18;line[i];i++)
+          if(line[i]=='\r' || line[i]=='\n')
+             line[i]=0;
+
+       password=&line[18];
+      }
+
+    if(strcmp(password,PassWord))
+      {
+       char *msg="WWWOFFLE Incorrect Password\n";
+       write(fileno(client),msg,strlen(msg));
+
+       PrintMessage(Warning,"Incorrect Password.");
+
+       return;
+      }
+
+    if(!(line=fgets_realloc(line,client)))
+      {PrintMessage(Warning,"Unexpected end of wwwoffle control command [%!s]."); return;}
+   }
+
  if(!strncmp(&line[9],"ONLINE",6))
    {
-    char *msg="WWWOFFLE Now Online\n";
+    char *msg;
 
-    PrintMessage(Inform,"WWWOFFLE Online.");
+    if(online)
+       msg="WWWOFFLE Already Online\n";
+    else
+       msg="WWWOFFLE Now Online\n";
 
     write(fileno(client),msg,strlen(msg));
 
+    if(!online)
+       PrintMessage(Important,"WWWOFFLE Online.");
+
     online=1;
    }
  else if(!strncmp(&line[9],"OFFLINE",7))
    {
-    char *msg="WWWOFFLE Now Offline\n";
+    char *msg;
 
-    PrintMessage(Inform,"WWWOFFLE Offline.");
+    if(!online)
+       msg="WWWOFFLE Already Offline\n";
+    else
+       msg="WWWOFFLE Now Offline\n";
 
     write(fileno(client),msg,strlen(msg));
 
+    if(online)
+       PrintMessage(Important,"WWWOFFLE Offline.");
+
     online=0;
 
-    if(fetching)
-      {
-       close(fetching);
-       fetching=0;
-      }
+    if(fetch_fd>0)
+       fetch_fd=-fetch_fd;
    }
  else if(!strncmp(&line[9],"FETCH",5))
    {
     char *msg;
 
-    PrintMessage(Inform,"WWWOFFLE Fetch.");
-
-    if(fetching)
+    if(fetch_fd)
        msg="WWWOFFLE Already fetching.\n";
     else if(!online)
        msg="WWWOFFLE Must be online to fetch.\n";
@@ -94,62 +143,55 @@ void CommandConnect(FILE *client)
 
     write(fileno(client),msg,strlen(msg));
 
-    if(!fetching && online)
+    PrintMessage(Important,"WWWOFFLE Fetch.");
+
+    if(!fetch_fd && online)
       {
-       int i;
+       fetch_fd=fileno(client);
 
-       fetching=fileno(client);
-
-       for(i=0;i<(MAX_SERVERS/2) && n_servers<=MAX_SERVERS;i++)
-          ForkServer(fetching,0);
+       for(;n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS;)
+          ForkServer(fetch_fd,0);
       }
    }
- else if(!strncmp(&line[9],"PURGE ",6))
+ else if(!strncmp(&line[9],"CONFIG",6))
    {
-    char msg[80];
-    int age=DEF_AGE;
-    char **hosts=NULL;
-    int nhosts=0;
-
-    PrintMessage(Inform,"WWWOFFLE Purge.");
-
-    do
-      {
-       while(line[strlen(line)-1]=='\r' || line[strlen(line)-1]=='\n')
-          line[strlen(line)-1]=0;
-
-       if(!strncmp(&line[15],"AGE ",4))
-         {
-          char ages[10];
-          int tempage=atoi(&line[19]);
-
-          sprintf(ages,"%d",tempage);
-
-          if(!strcmp(&line[19],ages))
-             age=tempage;
-         }
-       else if(!strncmp(&line[15],"HOST ",5))
-         {
-          if(!hosts)
-             hosts=(char**)malloc(8*sizeof(char*));
-          else if(!(nhosts%8))
-             hosts=realloc(hosts,(8+nhosts)*sizeof(char*));
-
-          hosts[nhosts]=(char*)malloc(strlen(line)-12);
-          strcpy(hosts[nhosts],&line[20]);
-          nhosts++;
-         }
-      }
-    while((line=fgets_realloc(line,client)) && line[1] && line[2]);
-
-    sprintf(msg,"WWWOFFLE Purging pages older than %d days from %s hosts.\n",age,hosts?"specified":"all");
+    char *msg="WWWOFFLE Reading Configuration File.\n";
     write(fileno(client),msg,strlen(msg));
 
-    PurgeCache(age,nhosts,hosts,fileno(client));
+    PrintMessage(Important,"WWWOFFLE Re-reading Configuration File.");
 
-    for(nhosts--;nhosts>=0;nhosts--)
-       free(hosts[nhosts]);
-    free(hosts);
+    if(ReadConfigFile(fileno(client)))
+      {
+       PrintMessage(Warning,"Error in configuration file; keeping old values.");
+
+       msg="WWWOFFLE Error Reading Configuration File.\n";
+      }
+    else
+       msg="WWWOFFLE Read Configuration File.\n";
+
+    write(fileno(client),msg,strlen(msg));
+
+    PrintMessage(Important,"WWWOFFLE Finished Re-reading Configuration File.");
+   }
+ else if(!strncmp(&line[9],"PURGE",5))
+   {
+    char *msg="WWWOFFLE Purge Starting.\n";
+    write(fileno(client),msg,strlen(msg));
+
+    if(PurgeUseMTime)
+       msg="(Using modification time.)\n";
+    else
+       msg="(Using last access time.)\n";
+    write(fileno(client),msg,strlen(msg));
+
+    PrintMessage(Important,"WWWOFFLE Purge.");
+
+    PurgeCache(fileno(client));
+
+    msg="WWWOFFLE Purge Finished.\n";
+    write(fileno(client),msg,strlen(msg));
+
+    PrintMessage(Important,"WWWOFFLE Purge finished.");
    }
  else
    {
@@ -178,22 +220,8 @@ void CommandConnect(FILE *client)
 
 void ForkServer(int client,int browser)
 {
- char *command[5],clientstr[5];
  int pid;
 
- sprintf(clientstr,"%d",client);
-
- command[0]="./wwwoffles";
- if(online && browser)
-    command[1]="-real";
- else if(online && !browser)
-    command[1]="-fetch";
- else if(!online && browser)
-    command[1]="-spool";
- command[2]=clientstr;
- command[3]=proxy;
- command[4]=NULL;
-
  if((pid=fork())==-1)
     PrintMessage(Warning,"Cannot fork a server [%!s].");
  else
@@ -201,20 +229,33 @@ void ForkServer(int client,int browser)
     if(pid)                      /* The parent */
       {
        n_servers++;
-       PrintMessage(Inform,"Forked wwwoffles %s (pid=%d).",command[1],pid);
+
+       if(online && !browser)
+         {
+          int i;
+
+          for(i=0;i<MAX_FETCH_SERVERS;i++)
+             if(fetch_pids[i]==0)
+               {fetch_pids[i]=pid;break;}
+
+          n_fetch_servers++;
+         }
+
+       PrintMessage(Inform,"Forked wwwoffles -%s (pid=%d).",online?browser?"real":"fetch":"spool",pid);
       }
     else                         /* The child */
       {
-       if(fetching && fetching!=client)
-          close(fetching);
+       int status;
+
+       if(fetch_fd && fetch_fd!=client)
+          close(fetch_fd);
 
        close(http_fd);
        close(wwwoffle_fd);
 
-       execv(command[0],command);
+       status=wwwoffles(online,browser,client);
 
-       PrintMessage(Warning,"Can not execv ./wwwoffles [%!s].");
-       exit(1);
+       exit(status);
       }
    }
 }
