@@ -1,8 +1,8 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.c 1.3 1997/01/25 12:38:18 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.c 1.7 1997/03/04 20:33:24 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
-  Generate error messages in a standard format on stderr.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  Generate error messages in a standard format on stderr and optionally also to syslog.
   ******************/ /******************
   Written by Andrew M. Bishop
 
@@ -18,6 +18,11 @@
 #define USE_STD_ARG
 #endif
 
+/*+ Under Linux use syslog as an option for error messages. +*/
+#if defined(__linux__)
+#define USE_SYSLOG
+#endif
+
 #include <unistd.h>
 
 #include <stdio.h>
@@ -30,6 +35,10 @@
 #include <varargs.h>
 #endif
 
+#ifdef USE_SYSLOG
+#include <syslog.h>
+#endif
+
 #include <errno.h>
 
 /* SunOS 4.x does not have strerror(). */
@@ -46,27 +55,45 @@ char* strerror(int err)
 }
 #endif
 
+#include "config.h"
 #include "errors.h"
 
 
 /*+ The name of the program. +*/
-static char *program;
+static char *program=NULL;
 
 /*+ The process id of the program. +*/
 static pid_t pid;
 
 /*+ The error messages. +*/
-static char *ErrorString[]={"","Warning: ","Fatal: "};
+static char *ErrorString[]={" Debug:",""      ,""        ," Warning:"," Fatal:"};
+
+#ifdef USE_SYSLOG
+static int use_syslog=0;
+
+/*+ The priority to apply to syslog messages. +*/
+static int ErrorPriority[]={       -1,      -1,LOG_NOTICE,LOG_WARNING,LOG_ERR};
+#endif
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Initialise the error handler, get the program name and pid.
 
   char *name The name of the program.
+
+  int syslogable Set to true if the errors are allowed to go to syslog.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void InitErrorHandler(char *name)
+void InitErrorHandler(char *name,int syslogable)
 {
+#ifdef USE_SYSLOG
+ use_syslog=syslogable;
+
+ if(program) closelog();
+ openlog(name,LOG_CONS|LOG_PID,LOG_DAEMON);
+ atexit(closelog);
+#endif
+
  program=name;
 
  pid=getpid();
@@ -76,62 +103,85 @@ void InitErrorHandler(char *name)
 /*++++++++++++++++++++++++++++++++++++++
   Print an error message.
 
-  int errlev Which error level.
+  ErrorLevel errlev Which error level.
 
   char* fmt The format of the message.
 
   ... The rest of the arguments (printf style).
   ++++++++++++++++++++++++++++++++++++++*/
 
-void PrintMessage(int errlev,char* fmt, ...)
+void PrintMessage(ErrorLevel errlev,char* fmt, ...)
 {
- int str_len=strlen(program)+strlen(ErrorString[errlev])+strlen(fmt)+16;
- char* string=(char*)malloc(str_len);
+ int str_len=16+strlen(fmt);
+ char* string;
  va_list ap;
  int i,j;
 
+#if !DEBUG
+ if(errlev==Debug)
+    return;
+#endif
+
+ string=(char*)malloc(str_len);
+
 #ifdef USE_STD_ARG
  va_start(ap,fmt);
 #else
  va_start(ap);
 #endif
 
- sprintf(string,"%s[%d] %s",program,pid,ErrorString[errlev]);
-
- for(i=0,j=strlen(string);fmt[i];i++)
-   {
+ for(i=0,j=0;fmt[i];i++)
     if(fmt[i]!='%')
        string[j++]=fmt[i];
     else
-       if(fmt[++i]!='!')
-         {
-          string[j++]='%';
-          string[j++]=fmt[i];
-         }
-       else
-         {
-          char buf[8],*msg=buf;
+      {
+       char str[16],*strp=NULL;
 
+       switch(fmt[++i])
+         {
+         case '!':
           if(fmt[++i]=='s')
-             msg=strerror(errno);
+             strp=strerror(errno);
           else
-             sprintf(buf,"%d",errno);
+             sprintf(strp=str,"%d",errno);
+          break;
 
-          str_len+=strlen(msg);
-          string=realloc(string,str_len);
-          string[j]=0;
-          strcat(string,msg);
-          j+=strlen(msg);
+         case 'd':
+          sprintf(strp=str,"%d",va_arg(ap,int));
+          break;
+
+         case 's':
+          strp=va_arg(ap,char*);
+          if(!strp) strp="(null)";
+          break;
+
+         default:
+          str[0]='%';
+          str[1]=fmt[i];
+          str[2]=0;
+          strp=str;
+          (void)va_arg(ap,void*);
          }
-   }
+
+       str_len+=strlen(strp);
+       string=realloc(string,str_len);
+       strcpy(&string[j],strp);
+       j+=strlen(strp);
+      }
+
  if(string[j-1]!='\n')
     string[j++]='\n';
  string[j]=0;
 
- vfprintf(stderr,string,ap);
-
  va_end(ap);
 
+#ifdef USE_SYSLOG
+ if(UseSyslog && use_syslog && ErrorPriority[errlev]!=-1)
+    syslog(ErrorPriority[errlev],"%s",string);
+#endif
+
+ fprintf(stderr,"%s[%d]%s %s",program,pid,ErrorString[errlev],string);
+
  free(string);
 
  if(errlev==Fatal)
