@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/io.c 1.4 1997/01/25 12:38:23 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/io.c 1.5 1997/02/19 21:49:05 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   Functions for file input and output.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -22,6 +22,7 @@
 #include <unistd.h>
 
 #include "wwwoffle.h"
+#include "errors.h"
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -91,9 +92,7 @@ int ReadOrTimeout(int fd,char *buffer,int n)
  nr=read(fd,buffer,n);          /* This can return -1 due to EPIPE !! */
  if(nr==-1 && errno==EPIPE)
    {
-/*
-    PrintMessage(0,"EPIPE problem - this cannot happen.");
-*/
+    PrintMessage(Debug,"EPIPE problem - this cannot happen.");
     sleep(1);                   /* This seems to cure the problem. */
     nr=read(fd,buffer,n);
    }
