@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/purge.c 1.4 1997/01/25 12:38:30 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/purge.c 1.8 1997/03/16 17:30:43 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   Purge old files from the cache.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -24,44 +24,27 @@
 #include <unistd.h>
 
 #include "wwwoffle.h"
+#include "config.h"
 #include "errors.h"
 
 
-static int PurgeFiles(char *dirname);
-
-/*+ The time of the oldest file to keep. +*/
-static time_t old=0;
+/* Local functions */
+static int PurgeFiles(char *dirname,int oldest);
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Purge files from the cache that meet the criteria.
+  Purge files from the cache that meet the age criteria.
 
-  int age The maximum age of files to keep (in days).
-
-  int nhosts The number of host name that have been specified.
-
-  char **hosts The list of host names that have to be purged.
-
-  int client the file descriptor of the wwwoffle client.
+  int fd the file descriptor of the wwwoffle client.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void PurgeCache(int age,int nhosts,char **hosts,int client)
+void PurgeCache(int fd)
 {
  DIR *dir;
  struct dirent* ent;
- int i;
- time_t now=0;
-
- /* Determine the age of old files. */
-
- now=time(NULL);
-
- if(age>0)
-    old=now-age*3600*24;
- else if(age<0)
-   {PrintMessage(Warning,"Purge age must be positive; purge failed.");return;}
- else
-    old=now+3600;
+ time_t now=time(NULL);
+ int total_bytes=0,total_dirs=0;
+ char *msg;
 
  /* Open the spool directory. */
 
@@ -69,46 +52,36 @@ void PurgeCache(int age,int nhosts,char **hosts,int client)
  if(!dir)
    {PrintMessage(Warning,"Cannot open spool directory [%!s]; purge failed.");return;}
 
- ent=readdir(dir);
+ ent=readdir(dir);  /* skip .  */
  if(!ent)
    {PrintMessage(Warning,"Cannot read spool directory [%!s]; purge failed.");closedir(dir);return;}
-
- ent=readdir(dir);  /* skip .  */
  ent=readdir(dir);  /* skip .. */
 
  /* Search through all of the sub directories. */
 
- while(ent)
+ while((ent=readdir(dir)))
    {
-    char *name=ent->d_name;
     struct stat buf;
 
-    if(stat(name,&buf))
-       PrintMessage(Warning,"Cannot stat file '%s' [%!s]; race condition?",name);
+    if(stat(ent->d_name,&buf))
+       PrintMessage(Inform,"Cannot stat file '%s' [%!s]; race condition?",ent->d_name);
     else
-       if(S_ISDIR(buf.st_mode))
+       if(S_ISDIR(buf.st_mode) && strcmp(ent->d_name,"outgoing"))
          {
-          int match=0;
+          int bytes=0;
+          int age=WhatPurgeAge(ent->d_name);
+          char *msg=(char*)malloc(strlen(ent->d_name)+80);
 
-          if(!nhosts)
-             match=1;
+          if(age<0)
+             bytes=PurgeFiles(ent->d_name,0);
           else
-             for(i=0;i<nhosts;i++)
-                if(!strcmp(name,hosts[i]))
-                  {match=1; *hosts[i]=0;}
-
-          if(match)
             {
-             char *msg=(char*)malloc(strlen(name)+16);
+             bytes=PurgeFiles(ent->d_name,now-age*24*3600+60);
 
-             sprintf(msg,"Purging %s\n",name);
-             write(client,msg,strlen(msg));
-             free(msg);
-
-             if(!PurgeFiles(name) && strcmp(name,"outgoing"))
+             if(!bytes)
                {
-                if(rmdir(name))
-                   PrintMessage(Warning,"Cannot delete 'empty' directory '%s' [%!s].",name);
+                if(rmdir(ent->d_name))
+                   PrintMessage(Warning,"Cannot delete 'empty' directory '%s' [%!s].",ent->d_name);
                }
              else
                {
@@ -116,38 +89,47 @@ void PurgeCache(int age,int nhosts,char **hosts,int client)
 
                 utbuf.actime=buf.st_atime;
                 utbuf.modtime=buf.st_mtime;
-                utime(name,&utbuf);
+                utime(ent->d_name,&utbuf);
                }
             }
-         }
 
-    ent=readdir(dir);
+          total_bytes+=bytes;
+          if(bytes)
+             total_dirs++;
+
+          if(age<0)
+             sprintf(msg,"Not Purged       %30s ; %4d kB.\n",ent->d_name,(bytes+512)/1024);
+          else if(bytes==0)
+             sprintf(msg,"Purged (%2d days) %30s ; empty.\n",age,ent->d_name);
+          else
+             sprintf(msg,"Purged (%2d days) %30s ; %4d kB.\n",age,ent->d_name,(bytes+512)/1024);
+          write(fd,msg,strlen(msg));
+          free(msg);
+         }
    }
 
  closedir(dir);
 
- for(i=0;i<nhosts;i++)
-    if(*hosts[i])
-      {
-       char *msg=(char*)malloc(strlen(hosts[i])+24);
-       sprintf(msg,"No such directory %s\n",hosts[i]);
-       write(client,msg,strlen(msg));
-       free(msg);
-      }
+ msg=(char*)malloc(64);
+ sprintf(msg,"Total of %d directories ; %d kB.\n",total_dirs,(total_bytes+512)/1024);
+ write(fd,msg,strlen(msg));
+ free(msg);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Delete the file in the current directory that are older than the specified age.
 
-  int PurgeFiles Returns the number of files that are left.
+  int PurgeFiles Returns the number of bytes in files that are left.
 
   char *dirname The name of the directory to purge.
+
+  int oldest The age of the oldest file to leave behind (in seconds).
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int PurgeFiles(char *dirname)
+static int PurgeFiles(char *dirname,int oldest)
 {
- int files_left=0;
+ int bytes_left=0;
  DIR *dir;
  struct dirent* ent;
 
@@ -160,37 +142,38 @@ static int PurgeFiles(char *dirname)
  if(!dir)
    {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not purged.",dirname);chdir("..");return(1);}
 
- ent=readdir(dir);
+ ent=readdir(dir);  /* skip .  */
  if(!ent)
    {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not purged.",dirname);closedir(dir);chdir("..");return(1);}
-
- ent=readdir(dir);  /* skip .  */
  ent=readdir(dir);  /* skip .. */
 
  /* Check all of the files for age, and delete as needed. */
 
- while(ent)
+ while((ent=readdir(dir)))
    {
-    char *name=ent->d_name;
     struct stat buf;
 
-    if(stat(name,&buf))
-       PrintMessage(Warning,"Cannot stat file '%s/%s' [%!s]; race condition?",dirname,name);
+    if(stat(ent->d_name,&buf))
+       PrintMessage(Inform,"Cannot stat file '%s/%s' [%!s]; race condition?",dirname,ent->d_name);
     else
       {
-       time_t t=buf.st_atime;
+       time_t t;
 
-       if(t>old)
-          files_left++;
+       if(PurgeUseMTime)
+          t=buf.st_mtime;
        else
-          if(unlink(name))
-             PrintMessage(Warning,"Cannot unlink file '%s/%s' [%!s].",dirname,name);
-      }
+          t=buf.st_atime;
 
-    ent=readdir(dir);
+       if(t>oldest)
+          bytes_left+=buf.st_size;
+       else
+          if(unlink(ent->d_name))
+             PrintMessage(Warning,"Cannot unlink file '%s/%s' [%!s].",dirname,ent->d_name);
+      }
    }
 
+ closedir(dir);
  chdir("..");
 
- return(files_left);
+ return(bytes_left);
 }
