@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/sockets.c 1.2 1997/01/25 12:37:23 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/sockets.c 1.3 1997/02/19 21:49:11 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   Socket manipulation routines.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -64,7 +64,7 @@ int OpenClientSocket(char* host, int port)
       {
        err=errno;
        if(--retries && errno==ECONNREFUSED)
-          PrintMessage(Inform,"Connect fail [%!s]; trying again");
+          PrintMessage(Inform,"Connect fail [%!s]; trying again.");
        else
           PrintMessage(Warning,"Connect fail [%!s].");
        close(s);
