@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/spool.c 1.3 1997/01/22 20:21:09 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/spool.c 1.9 1997/03/24 10:06:12 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   Handle all of the spooling of files in the spool directory.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -17,6 +17,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <utime.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <unistd.h>
@@ -38,8 +39,8 @@
 
 int OpenOutgoingSpoolFile(int rw)
 {
- int fd=-1;
- int high=0,low=0;
+ int fd=-2;
+ int low=0,high=0;
  DIR *dir;
  struct dirent* ent;
 
@@ -49,47 +50,41 @@ int OpenOutgoingSpoolFile(int rw)
  if(!dir)
    {
     PrintMessage(Inform,"Cannot open outgoing directory [%!s]; creating one.");
-    if(!mkdir("outgoing",0755))
-      {PrintMessage(Warning,"Cannot create outgoing directory [%!s].");return(-1);}
+    if(mkdir("outgoing",0755) || !(dir=opendir("outgoing")))
+      {PrintMessage(Warning,"Cannot create or access outgoing directory [%!s].");return(-1);}
    }
 
- ent=readdir(dir);
+ ent=readdir(dir);  /* skip .  */
  if(!ent)
    {PrintMessage(Warning,"Cannot read outgoing directory [%!s].");closedir(dir);return(-1);}
-
- ent=readdir(dir);  /* skip .  */
  ent=readdir(dir);  /* skip .. */
 
- /* Go through the files and find the highest/lowest number. */
+ /* Go through the files and find the lowest/highest numbers. */
 
- while(ent)
+ while((ent=readdir(dir)))
    {
     int number=atoi(ent->d_name);
 
     if(number)
       {
-       if(!high || number>high)
-          high=number;
        if(!low  || number<low)
           low=number;
+       if(!high || number>high)
+          high=number;
       }
-
-    ent=readdir(dir);
    }
 
  closedir(dir);
 
- /* Open the lowest to read or the highest to write. */
+ /* Open the lowest to read or a temporary one to write. */
 
  if(rw)
    {
-    /* Open the lowest number. */
-
     if(low)
-       for(;low<=high;low++) /* start at the lowest and keep going to the highest. */
+       for(;low<=(high+8);low++) /* start at the lowest and keep going to the highest. */
          {
           struct stat buf;
-          char name[20];
+          char name[24];
 
           sprintf(name,"outgoing/%d",low);
 
@@ -97,7 +92,10 @@ int OpenOutgoingSpoolFile(int rw)
             {
              fd=open(name,O_RDONLY);
              if(fd==-1)
+               {
                 PrintMessage(Inform,"Cannot open file '%s' [%!s]; race condition?",name);
+                fd=-2;
+               }
              else
                {
                 unlink(name);
@@ -107,43 +105,94 @@ int OpenOutgoingSpoolFile(int rw)
           else
              PrintMessage(Inform,"Cannot stat file '%s' [%!s]; race condition?",name);
          }
-    else
-       return(-2);
    }
  else
    {
-    /* Open the next highest number. */
+    char name[24];
 
-    for(high++;;high++)         /* start 1 higher and keep going */
-      {
-       struct stat buf;
-       char name[20];
+    sprintf(name,"outgoing/tmp.%d",getpid());
 
-       sprintf(name,"outgoing/%d",high);
-
-       if(stat(name,&buf))
-         {
-          fd=open(name,O_WRONLY|O_CREAT|O_EXCL,0644);
-          if(fd==-1)
-             PrintMessage(Inform,"Cannot open file '%s' [%!s]; race condition?",name);
-          else
-             break;
-         }
-       else
-          PrintMessage(Inform,"Can stat file '%s'; race condition?",name);
-      }
+    fd=open(name,O_WRONLY|O_CREAT|O_EXCL,0644);
+    if(fd==-1)
+       PrintMessage(Inform,"Cannot open file '%s' [%!s]",name);
    }
 
  return(fd);
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Close the temporary outgoing spool file and rename it.
+
+  int fd The file descriptor that was being written to.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void CloseOutgoingSpoolFile(int fd)
+{
+ int high=0;
+ DIR *dir;
+ struct dirent* ent;
+ char tmpname[24];
+
+ close(fd);
+
+ /* Open the outgoing directory. */
+
+ dir=opendir("outgoing");
+ if(!dir)
+   {
+    PrintMessage(Inform,"Cannot open outgoing directory [%!s]; creating one.");
+    if(mkdir("outgoing",0755) || !(dir=opendir("outgoing")))
+      {PrintMessage(Warning,"Cannot create or access outgoing directory [%!s].");return;}
+   }
+
+ ent=readdir(dir);  /* skip .  */
+ if(!ent)
+   {PrintMessage(Warning,"Cannot read outgoing directory [%!s].");closedir(dir);return;}
+ ent=readdir(dir);  /* skip .. */
+
+ /* Go through the files and find the highest number. */
+
+ while((ent=readdir(dir)))
+   {
+    int number=atoi(ent->d_name);
+
+    if(number)
+       if(!high || number>high)
+          high=number;
+   }
+
+ closedir(dir);
+
+ /* Rename the temporary file to the new file. */
+
+ sprintf(tmpname,"outgoing/tmp.%d",getpid());
+
+ for(high++;;high++)         /* start 1 higher and keep going */
+   {
+    struct stat buf;
+    char name[24];
+
+    sprintf(name,"outgoing/%d",high);
+
+    if(stat(name,&buf))
+      {
+       if(rename(tmpname,name))
+          PrintMessage(Inform,"Cannot rename file '%s' to '%s' [%!s].",tmpname,name);
+       break;
+      }
+    else
+       PrintMessage(Inform,"Can stat file '%s'; race condition?",name);
+   }
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Open a file in a spool subdirectory to write into / read from / delete.
 
   int OpenWebpageSpoolFile Returns a file descriptor.
 
-  int rw Set to +ve to read, zero to write, -ve to delete.
+  int rw Set to 1 to read, 0 to write, -1 to delete, -2 to touch.
 
   char *host The hostname from the URL.
 
@@ -189,7 +238,7 @@ int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args)
 
  if(stat(host,&buf))
    {
-    PrintMessage(Inform,"Cannot open directory '%s' [%!s]; creating one.",host);
+    PrintMessage(Inform,"Directory '%s' does not exist [%!s]; creating one.",host);
     if(mkdir(host,0755))
       {PrintMessage(Warning,"Cannot create directory '%s' [%!s].",host);free(file);return(-1);}
    }
@@ -199,20 +248,20 @@ int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args)
 
  /* Create the file for the web page. */
 
- if(rw>0)
+ if(rw==0)
+    fd=open(file,O_RDWR|O_CREAT,0644);
+ else if(rw==1)
     fd=open(file,O_RDONLY);
- else if(rw==0)
-    fd=open(file,O_WRONLY|O_CREAT|O_TRUNC,0644);
- else
+ else if(rw==-1)
     unlink(file);
+ else if(rw==-2)
+    utime(file,NULL);
 
  if(args && (rw<0 || fd!=-1))
    {
     sprintf(file,"%s/%c%c%08lx",host,PATH_SEP,PATH_SEP,hash);
 
-    if(rw<0)
-       unlink(file);
-    else if(rw==0)
+    if(rw==0)
       {
        int afd=open(file,O_WRONLY|O_CREAT|O_TRUNC,0644);
        if(afd!=-1)
@@ -226,7 +275,7 @@ int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args)
           fd=-1;
          }
       }
-    else
+    else if(rw==1)
       {
        int afd=open(file,O_RDONLY);
        if(afd!=-1)
@@ -237,6 +286,10 @@ int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args)
           fd=-1;
          }
       }
+    else if(rw==-1)
+       unlink(file);
+    else if(rw==-2)
+       utime(file,NULL);
    }
 
  free(file);
@@ -244,31 +297,3 @@ int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args)
 
  return(fd);
 }
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Generate a hash value depending on the arguments appended to the URL.
-
-  unsigned long MakeHashFromArgs Returns a long integer to use.
-
-  char *args The arguments.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-unsigned long MakeHashFromArgs(char *args)
-{
- int i;
- unsigned long hash=0;
-
- if(args)
-    for(i=0;args[i];)
-      {
-       unsigned long subhash=args[i]&0x7f;
-
-       for(i++;i%25 && args[i];i++)
-          subhash=(subhash<<1)^(unsigned long)(args[i]&0x7f);
-
-       hash^=subhash;
-      }
-
- return(hash);
-}
