@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.4 1997/01/22 20:21:58 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.17 1997/03/24 17:36:51 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   A header file for all of the programs wwwoffle, wwwoffled, wwwoffles.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -28,11 +28,14 @@
 #define DEF_SPOOL "/var/spool/wwwoffle"
 
 /*+ The default age of pages in days before purging. +*/
-#define DEF_AGE 14
+#define DEF_PURGE_AGE 14
 
-/*+ The maximum number of servers to fork. +*/
+/*+ The maximum number of servers to fork in total. +*/
 #define MAX_SERVERS 8
 
+/*+ The maximum number of servers to fork for fetching previously requested pages. +*/
+#define MAX_FETCH_SERVERS 4
+
 /*+ The character to use to replace '/' in the spool directories. +*/
 #define PATH_SEP '%'
 
@@ -46,35 +49,62 @@ void ForkServer(int client,int browser);
 
 /* In purge.c */
 
-void PurgeCache(int age,int nhosts,char **hosts,int client);
+void PurgeCache(int fd);
 
 /* In spool.c */
 
 int OpenOutgoingSpoolFile(int rw);
+void CloseOutgoingSpoolFile(int fd);
 int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args);
-unsigned long MakeHashFromArgs(char *args);
 
 /* In parse.c */
 
 char *ParseRequest(FILE *req,char **url);
-char *ParseReply(int rep,int *status);
+int RequestChanges(int fd,char **request);
+char *RequestMoved(char *request,char *reply);
+char *RequestURL(char *url,char *referer);
+void MakeRequestNonProxy(char *url,char *path,char *args,char **request);
+char *ParseReply(int rep,int *status,int *length);
 void SplitURL(char *url,char **host,char **path,char **args);
-void MakeRequestNonProxy(char *url,char *path,char *args,char **buffer);
+unsigned long MakeHashFromArgs(char *args);
+char *RFC822Date(long t);
 
 /* In messages.c */
 
-void ServerErrorMessage(int fd,char *string);
-void ServerFetchFailed(int fd,char *url,char *host,int port,int connect_or_timeout);
-void WillGet(int fd,char *url,int already);
-void Refresh(int fd,char *url);
+void ServerError(int fd,char *string);
+void RemoteHostError(int fd,char *url,char *host,int port,char *reason);
+void WillGetURL(int fd,char *url,int already);
+void RefreshRedirect(int fd,char *url);
+void WelcomePage(int fd,int password);
+void IllegalPage(int fd,char *path,char *args);
 
 /* In index.c */
 
-void GenerateIndex(int fd,char *path,char *args);
+void IndexPage(int fd,char *path,char *args);
+
+/* In control.c */
+
+void ControlPage(int fd,char *path,char *request);
+
+/* In refresh.c */
+
+void RefreshPage(int fd,char *path,char *request);
+void RecurseFetchPages(int fd,char *url,int images,int recursive,int recursive_any,char *host,int port);
+
+/* In html.c (html.l) */
+
+int ParseHTML(FILE *file,char *host,char *path);
+char **ListImages(void);
+char **ListLinks(void);
+int GetHTMLEnd(void);
 
 /* In io.c */
 
 char *fgets_realloc(char *buffer,FILE *file);
 int ReadOrTimeout(int fd,char *buffer,int n);
 
+/* In wwwoffles.c */
+
+int wwwoffles(int online,int browser,int client);
+
 #endif /* WWWOFFLE_H */
