@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.4 1997/01/22 20:22:33 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.11 1997/03/24 10:06:13 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -26,6 +26,7 @@
 #include <signal.h>
 
 #include "wwwoffle.h"
+#include "config.h"
 #include "sockets.h"
 #include "errors.h"
 
@@ -35,9 +36,6 @@ static void sigchild(int signum);
 static void sigexit(int signum);
 
 
-/*+ The proxy server that we use. +*/
-char *proxy=NULL;
-
 /*+ The server sockets that we listen on +*/
 int http_fd=-1,                 /*+ for the HTTP connections. +*/
     wwwoffle_fd=-1;             /*+ for the WWWOFFLE connections. +*/
@@ -48,11 +46,15 @@ static int closedown=0;
 /*+ The online / offline status. +*/
 int online=0;
 
-/*+ The current number of active servers. +*/
-int n_servers=0;
+/*+ The current number active servers +*/
+int n_servers=0,                /*+ in total. +*/
+    n_fetch_servers=0;          /*+ fetching a page. +*/
 
-/*+ The wwwoffle client when fetching. +*/
-int fetching=0;
+/*+ The pids of the servers that are fetching. +*/
+int fetch_pids[MAX_FETCH_SERVERS];
+
+/*+ The wwwoffle client file descriptor when fetching. +*/
+int fetch_fd=0;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -61,11 +63,6 @@ int fetching=0;
 
 int main(int argc, char** argv)
 {
- int port1=DEF_HTTP_PORT,
-     port2=DEF_WWWOFFLE_PORT;
-
- char *spool=DEF_SPOOL;
-
  int i;
  int err;
  struct stat buf;
@@ -77,40 +74,12 @@ int main(int argc, char** argv)
     if(!strcmp(argv[i],"-h"))
        usage(1);
 
-    if(!strcmp(argv[i],"-proxy"))
+    if(!strcmp(argv[i],"-c"))
       {
        if(++i>=argc)
-         {fprintf(stderr,"wwwoffled: The '-proxy' argument requires a hostname.\n"); exit(1);}
+         {fprintf(stderr,"wwwoffled: The '-c' option requires a filename.\n"); exit(1);}
 
-       proxy=argv[i];
-       continue;
-      }
-
-    if(!strcmp(argv[i],"-ports"))
-      {
-       if(i>=(argc-2))
-         {fprintf(stderr,"wwwoffled: The '-ports' argument requires two port numbers.\n"); exit(1);}
-
-       port1=atoi(argv[++i]);
-       if(port1<=0 || port1>=65536)
-         {fprintf(stderr,"wwwoffled: The port1 number '%s' is invalid.\n",argv[i]); exit(1);}
-
-       port2=atoi(argv[++i]);
-       if(port2<=0 || port2>=65536)
-         {fprintf(stderr,"wwwoffled: The port2 number '%s' is invalid.\n",argv[i]); exit(1);}
-
-       if(port1==port2)
-         {fprintf(stderr,"wwwoffled: The two port numbers are the same '%d'.\n",port1); exit(1);}
-
-       continue;
-      }
-
-    if(!strcmp(argv[i],"-spool"))
-      {
-       if(++i>=argc)
-         {fprintf(stderr,"wwwoffled: The '-spool' argument requires a spool directory.\n"); exit(1);}
-
-       spool=argv[i];
+       ConfigFile=argv[i];
        continue;
       }
 
@@ -119,35 +88,39 @@ int main(int argc, char** argv)
 
  /* Initialise things. */
 
- InitErrorHandler("wwwoffled");
+ for(i=0;i<MAX_FETCH_SERVERS;i++)
+    fetch_pids[i]=0;
 
- PrintMessage(Inform,"WWWOFFLE Demon Version 1.0 starting");
+ InitErrorHandler("wwwoffled",1);
 
- http_fd=OpenServerSocket(port1);
+ if(ReadConfigFile(2))
+    PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
+
+ PrintMessage(Important,"WWWOFFLE Demon Version 1.1 started");
+ PrintMessage(Inform,"WWWOFFLE Read Configuration File.");
+
+ http_fd=OpenServerSocket(HTTP_Port);
  if(http_fd==-1)
-    PrintMessage(Fatal,"Cannot create HTTP server.");
+    PrintMessage(Fatal,"Cannot create HTTP server socket.");
 
- wwwoffle_fd=OpenServerSocket(port2);
+ wwwoffle_fd=OpenServerSocket(WWWOFFLE_Port);
  if(wwwoffle_fd==-1)
-    PrintMessage(Fatal,"Cannot create WWWOFFLE server.");
+    PrintMessage(Fatal,"Cannot create WWWOFFLE server socket.");
 
- if(stat(spool,&buf))
+ if(stat(SpoolDir,&buf))
    {
-    err=mkdir(spool,0755);
+    err=mkdir(SpoolDir,0755);
     if(err==-1 && errno!=EEXIST)
-       PrintMessage(Fatal,"Cannot create spool directory %s [%!s].\n",spool);
+       PrintMessage(Fatal,"Cannot create spool directory %s [%!s].",SpoolDir);
+    stat(SpoolDir,&buf);
    }
 
- if(!stat(spool,&buf) && !S_ISDIR(buf.st_mode))
-    PrintMessage(Fatal,"The spool directory %s is not a directory.\n",spool);
+ if(!S_ISDIR(buf.st_mode))
+    PrintMessage(Fatal,"The spool directory %s is not a directory.",SpoolDir);
 
- err=chdir(spool);
+ err=chdir(SpoolDir);
  if(err==-1)
-    PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].\n",spool);
-
- err=mkdir("outgoing",0755);
- if(err==-1 && errno!=EEXIST)
-    PrintMessage(Fatal,"Cannot create spool 'outgoing' directory [%!s].\n");
+    PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].",SpoolDir);
 
  sigchild(0);
  sigexit(0);
@@ -184,22 +157,33 @@ int main(int argc, char** argv)
           client=AcceptConnect(wwwoffle_fd,&host,&port);
           if(client>=0)
             {
-             FILE *file=fdopen(client,"r");
+             int allowed=IsAllowedConnect(host);
 
-             if(!file)
+             if(allowed)
                {
-                PrintMessage(Warning,"Unable to fdopen the wwwoffle control socket [%!s].");
+                FILE *file=fdopen(client,"r");
 
-                CloseSocket(client);
+                PrintMessage(Important,"WWWOFFLE connection from %s.",host);
+
+                if(!file)
+                  {
+                   PrintMessage(Warning,"Unable to fdopen the wwwoffle control socket [%!s].");
+
+                   CloseSocket(client);
+                  }
+                else
+                  {
+                   CommandConnect(file);
+
+                   if(fetch_fd!=client)
+                      fclose(file);
+                  }
                }
              else
                {
-                PrintMessage(Inform,"WWWOFFLE connection from %s.",host);
+                PrintMessage(Warning,"WWWOFFLE connection from %s rejected.",host);
 
-                CommandConnect(file);
-
-                if(fetching!=client)
-                   fclose(file);
+                CloseSocket(client);
                }
             }
          }
@@ -212,9 +196,16 @@ int main(int argc, char** argv)
           client=AcceptConnect(http_fd,&host,&port);
           if(client>=0)
             {
-             PrintMessage(Inform,"HTTP Proxy connection from %s.",host);
+             int allowed=IsAllowedConnect(host);
 
-             ForkServer(client,1);
+             if(allowed)
+               {
+                PrintMessage(Inform,"HTTP Proxy connection from %s.",host);
+
+                ForkServer(client,1);
+               }
+             else
+                PrintMessage(Warning,"HTTP Proxy connection from %s rejected.",host);
 
              CloseSocket(client);
             }
@@ -241,7 +232,7 @@ int main(int argc, char** argv)
 static void usage(int verbose)
 {
  fputs("\n"
-       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.0\n"
+       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.1\n"
        "\n",stderr);
 
  if(verbose)
@@ -249,21 +240,16 @@ static void usage(int verbose)
           "                             [http://www.gedanken.demon.co.uk/]\n"
           "\n",stderr);
 
- fputs("Usage: wwwoffled [-h] [-proxy host[:port]] [-ports port1 port2] [-spool dir]\n"
+ fputs("Usage: wwwoffled [-h] [-c <config-file>]\n"
        "\n",stderr);
 
  if(verbose)
     fprintf(stderr,
             "   -h                  : Display this help.\n"
             "\n"
-            "   -proxy host[:port]  : The host name and port number of the proxy to use.\n"
-            "                         (Default no proxy).\n"
-            "   -ports port1 port2  : The port numbers to listen to;\n"
-            "                         first for HTTP and second for wwwoffle\n"
-            "                         (Defaults %d and %d).\n"
-            "   -spool dir          : The directory name to spool to.\n"
+            "   -c <config-file>    : The name of the configuration file to use.\n"
             "                         (Default %s).\n"
-            "\n",DEF_HTTP_PORT,DEF_WWWOFFLE_PORT,DEF_SPOOL);
+            "\n",ConfigFile);
 
  exit(0);
 }
@@ -284,6 +270,7 @@ static void sigchild(int signum)
  if(signum)
     while((pid=waitpid(-1,&status,WNOHANG))>0)
       {
+       int i;
        int exitval=0;
 
        if(WIFEXITED(status))
@@ -301,30 +288,48 @@ static void sigchild(int signum)
        else
           PrintMessage(Inform,"Child wwwoffles finished (pid=%d).",pid);
 
-       if(fetching)
+       n_servers--;
+
+       for(i=0;i<MAX_FETCH_SERVERS;i++)
+          if(fetch_pids[i]==pid)
+             break;
+
+       if(i!=MAX_FETCH_SERVERS)
          {
-          if(exitval==3)
+          n_fetch_servers--;
+          fetch_pids[i]=0;
+
+          if(fetch_fd>0 && exitval==3)
+             fetch_fd=-fetch_fd;
+
+          if(fetch_fd<0 && exitval==4)
+             fetch_fd=-fetch_fd;
+
+          if(fetch_fd<0 && n_fetch_servers==0)
             {
              char *msg="WWWOFFLE No more to fetch.\n";
-             write(fetching,msg,strlen(msg));
-             close(fetching);
-             fetching=0;
+             write(-fetch_fd,msg,strlen(msg));
+
+             PrintMessage(Important,"WWWOFFLE Fetch finished.");
+
+             close(-fetch_fd);
+             fetch_fd=0;
             }
-          else
-             ForkServer(fetching,0);
          }
 
-       n_servers--;
+       if(fetch_fd>0)
+          for(;n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS;)
+             ForkServer(fetch_fd,0);
       }
 
 #if defined(SIGCHLD)
  sig=signal(SIGCHLD,sigchild);
  if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGGHLD handler.\n");
+    PrintMessage(Warning,"Cannot add SIGGHLD handler.");
 #else
  sig=signal(SIGCLD,sigchild);
  if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGGLD handler.\n");
+    PrintMessage(Warning,"Cannot add SIGGLD handler.");
 #endif
 }
 
@@ -343,7 +348,7 @@ static void sigexit(int signum)
    {
     closedown=1;
 
-    PrintMessage(Inform,"Exit signalled.");
+    PrintMessage(Important,"Exit signalled.");
    }
 
  sig=signal(SIGINT,sigexit);
